@@ -126,6 +126,7 @@ def cart2sphere(x, y, z):
     '''
     r = np.sqrt(x * x + y * y + z * z)
     theta = np.arccos(z / r)
+    theta = np.where(r > 0, theta, 0.)
     phi = np.arctan2(y, x)
     r, theta, phi = np.broadcast_arrays(r, theta, phi)
     return r, theta, phi
@@ -500,7 +501,8 @@ def lambert_equal_area_projection_polar(theta, phi):
        planar coordinates of points following mapping by Lambert's EAP.
     """
 
-    return 2 * np.repeat(np.sin(theta / 2), 2).reshape((theta.shape[0], 2)) * np.column_stack((np.cos(phi), np.sin(phi)))
+    return 2 * np.repeat(np.sin(theta / 2), 2).reshape((theta.shape[0], 2)) * \
+        np.column_stack((np.cos(phi), np.sin(phi)))
 
 
 def lambert_equal_area_projection_cart(x, y, z):
@@ -575,7 +577,6 @@ def euler_matrix(ai, aj, ak, axes='sxyz'):
     try:
         firstaxis, parity, repetition, frame = _AXES2TUPLE[axes]
     except (AttributeError, KeyError):
-        _ = _TUPLE2AXES[axes]
         firstaxis, parity, repetition, frame = axes
 
     i = firstaxis
@@ -616,7 +617,8 @@ def euler_matrix(ai, aj, ak, axes='sxyz'):
     return M
 
 
-def compose_matrix(scale=None, shear=None, angles=None, translate=None, perspective=None):
+def compose_matrix(scale=None, shear=None, angles=None, translate=None,
+                   perspective=None):
     """Return 4x4 transformation matrix from sequence of
     transformations.
 
@@ -686,8 +688,8 @@ def compose_matrix(scale=None, shear=None, angles=None, translate=None, perspect
 def decompose_matrix(matrix):
     """Return sequence of transformations from transformation matrix.
 
-    Code modified from the excellent work of Christoph Gohlke link provided here
-    http://www.lfd.uci.edu/~gohlke/code/transformations.py.html
+    Code modified from the excellent work of Christoph Gohlke link provided
+    here: http://www.lfd.uci.edu/~gohlke/code/transformations.py.html
 
     Parameters
     ------------
@@ -767,7 +769,7 @@ def decompose_matrix(matrix):
         angles[0] = math.atan2(row[1, 2], row[2, 2])
         angles[2] = math.atan2(row[0, 1], row[0, 0])
     else:
-        #angles[0] = math.atan2(row[1, 0], row[1, 1])
+        # angles[0] = math.atan2(row[1, 0], row[1, 1])
         angles[0] = math.atan2(-row[2, 1], row[1, 1])
         angles[2] = 0.0
 
@@ -798,7 +800,8 @@ def circumradius(a, b, c):
     z = np.cross(x, y)
     # test for collinearity
     if np.linalg.norm(z) == 0:
-        return np.sqrt(np.max(np.dot(x, x), np.dot(y, y), np.dot(a - b, a - b))) / 2.
+        return np.sqrt(np.max(np.dot(x, x), np.dot(y, y),
+                              np.dot(a - b, a - b))) / 2.
     else:
         m = np.vstack((x, y, z))
         w = np.dot(np.linalg.inv(m.T), np.array([xx / 2., yy / 2., 0]))
@@ -808,7 +811,8 @@ def circumradius(a, b, c):
 def vec2vec_rotmat(u, v):
     r""" rotation matrix from 2 unit vectors
 
-    u,v being unit 3d vectors return a 3x3 rotation matrix R than aligns u to v.
+    u, v being unit 3d vectors return a 3x3 rotation matrix R than aligns u to
+    v.
 
     In general there are many rotations that will map u to v. If S is any
     rotation using v as an axis then R.S will also map u to v since (S.R)u =
@@ -839,16 +843,25 @@ def vec2vec_rotmat(u, v):
     array([ 1.,  0.,  0.])
 
     """
-    norm_u_v = np.linalg.norm(u - v)
-    # return eye when u is the same with v
-    if np.linalg.norm(u - v) < np.finfo(float).eps:
-        return np.eye(3)
-    # This is the case of two antipodal vectors:
-    if norm_u_v == 2.0:
-        return -np.eye(3)
 
+    # Cross product is the first step to find R
+    # Rely on numpy instead of manual checking for failing
+    # cases
     w = np.cross(u, v)
-    w = w / np.linalg.norm(w)
+    wn = np.linalg.norm(w)
+
+    # Check that cross product is OK and vectors
+    # u, v are not collinear (norm(w)>0.0)
+    if np.isnan(wn) or wn < np.finfo(float).eps:
+        norm_u_v = np.linalg.norm(u - v)
+        # This is the case of two antipodal vectors:
+        # ** former checking assumed norm(u) == norm(v)
+        if norm_u_v > np.linalg.norm(u):
+            return -np.eye(3)
+        return np.eye(3)
+
+    # if everything ok, normalize w
+    w = w / wn
 
     # vp is in plane of u,v,  perpendicular to u
     vp = (v - (np.dot(u, v) * u))
@@ -863,15 +876,10 @@ def vec2vec_rotmat(u, v):
     Rp = np.dot(Pt, np.dot(R, P))
 
     # make sure that you don't return any Nans
-    if np.sum(np.isnan(Rp)) > 0:
+    # check using the appropriate tool in numpy
+    if np.any(np.isnan(Rp)):
         return np.eye(3)
 
-    # Everything's fine, up to a sign reversal:
-    rot_back = np.dot(Rp, v)
-    sign_reverser = np.sign((np.sign(rot_back) == np.sign(u)) - 0.5).squeeze()
-    # Multiply each line by it's reverser and reassemble the matrix:
-    Rp = Rp * sign_reverser[:, np.newaxis]
-
     return Rp
 
 
@@ -901,3 +909,119 @@ def compose_transformations(*mats):
 
     return prev
 
+
+def perpendicular_directions(v, num=30, half=False):
+    r""" Computes n evenly spaced perpendicular directions relative to a given
+    vector v
+
+    Parameters
+    -----------
+    v : array (3,)
+        Array containing the three cartesian coordinates of vector v
+    num : int, optional
+        Number of perpendicular directions to generate
+    half : bool, optional
+        If half is True, perpendicular directions are sampled on half of the
+        unit circumference perpendicular to v, otherwive perpendicular
+        directions are sampled on the full circumference. Default of half is
+        False
+
+    Returns
+    -------
+    psamples : array (n, 3)
+        array of vectors perpendicular to v
+
+    Notes
+    --------
+    Perpendicular directions are estimated using the following two step
+    procedure:
+
+        1) the perpendicular directions are first sampled in a unit
+        circumference parallel to the plane normal to the x-axis.
+
+        2) Samples are then rotated and aligned to the plane normal to vector
+        v. The rotational matrix for this rotation is constructed as reference
+        frame basis which axis are the following:
+            - The first axis is vector v
+            - The second axis is defined as the normalized vector given by the
+            cross product between vector v and the unit vector aligned to the
+            x-axis
+            - The third axis is defined as the cross product between the
+            previous computed vector and vector v.
+
+    Following this two steps, coordinates of the final perpendicular directions
+    are given as:
+
+    .. math::
+
+        \left [ -\sin(a_{i}) \sqrt{{v_{y}}^{2}+{v_{z}}^{2}}
+        \; , \;
+        \frac{v_{x}v_{y}\sin(a_{i})-v_{z}\cos(a_{i})}
+        {\sqrt{{v_{y}}^{2}+{v_{z}}^{2}}}
+        \; , \;
+        \frac{v_{x}v_{z}\sin(a_{i})-v_{y}\cos(a_{i})}
+        {\sqrt{{v_{y}}^{2}+{v_{z}}^{2}}} \right  ]
+
+    This procedure has a singularity when vector v is aligned to the x-axis. To
+    solve this singularity, perpendicular directions in procedure's step 1 are
+    defined in the plane normal to y-axis and the second axis of the rotated
+    frame of reference is computed as the normalized vector given by the cross
+    product between vector v and the unit vector aligned to the y-axis.
+    Following this, the coordinates of the perpendicular directions are given
+    as:
+
+        \left [ -\frac{\left (v_{x}v_{y}\sin(a_{i})+v_{z}\cos(a_{i}) \right )}
+        {\sqrt{{v_{x}}^{2}+{v_{z}}^{2}}}
+        \; , \;
+        \sin(a_{i}) \sqrt{{v_{x}}^{2}+{v_{z}}^{2}}
+        \; , \;
+        \frac{v_{y}v_{z}\sin(a_{i})+v_{x}\cos(a_{i})}
+        {\sqrt{{v_{x}}^{2}+{v_{z}}^{2}}} \right  ]
+
+    For more details on this calculation, see ` here <http://gsoc2015dipydki.blogspot.it/2015/07/rnh-post-8-computing-perpendicular.html>`_.
+    """
+    v = np.array(v, dtype=float)
+
+    # Float error used for floats comparison
+    er = np.finfo(v[0]).eps * 1e3
+
+    # Define circumference or semi-circumference
+    if half is True:
+        a = np.linspace(0., math.pi, num=num, endpoint=False)
+    else:
+        a = np.linspace(0., 2 * math.pi, num=num, endpoint=False)
+
+    cosa = np.cos(a)
+    sina = np.sin(a)
+
+    # Check if vector is not aligned to the x axis
+    if abs(v[0] - 1.) > er:
+        sq = np.sqrt(v[1]**2 + v[2]**2)
+        psamples = np.array([- sq*sina, (v[0]*v[1]*sina - v[2]*cosa) / sq,
+                             (v[0]*v[2]*sina + v[1]*cosa) / sq])
+    else:
+        sq = np.sqrt(v[0]**2 + v[2]**2)
+        psamples = np.array([- (v[2]*cosa + v[0]*v[1]*sina) / sq, sina*sq,
+                             (v[0]*cosa - v[2]*v[1]*sina) / sq])
+
+    return psamples.T
+
+
+def dist_to_corner(affine):
+    """Calculate the maximal distance from the center to a corner of a voxel,
+    given an affine
+
+    Parameters
+    ----------
+    affine : 4 by 4 array.
+        The spatial transformation from the measurement to the scanner space.
+
+    Returns
+    -------
+    dist: float
+        The maximal distance to the corner of a voxel, given voxel size encoded
+        in the affine.
+    """
+    R = affine[0:3, 0:3]
+    vox_dim = np.diag(np.linalg.cholesky(R.T.dot(R)))
+    return np.sqrt(np.sum((vox_dim / 2) ** 2))
