@@ -26,6 +26,9 @@ class GradientTable(object):
         diffusion gradients
     bvals : (N,) ndarray
         The b-value, or magnitude, of each gradient direction.
+    qvals: (N,) ndarray
+        The q-value for each gradient direction. Needs big and small
+        delta.
     bvecs : (N,3) ndarray
         The direction, represented as a unit vector, of each gradient.
     b0s_mask : (N,) ndarray
@@ -57,6 +60,11 @@ class GradientTable(object):
     def bvals(self):
         return vector_norm(self.gradients)
 
+    @auto_attr
+    def qvals(self):
+        tau = self.big_delta - self.small_delta / 3.0
+        return np.sqrt(self.bvals / tau) / (2 * np.pi)
+
     @auto_attr
     def b0s_mask(self):
         return self.bvals <= self.b0_threshold
@@ -217,7 +225,7 @@ def gradient_table(bvals, bvecs=None, big_delta=None, small_delta=None,
           _, bvecs = io.read_bvals_bvecs(None, bvecs)
 
     bvals = np.asarray(bvals)
-    # If bvals is None we expect bvals to be an (N, 3) or (3, N) array
+    # If bvecs is None we expect bvals to be an (N, 4) or (4, N) array.
     if bvecs is None:
         if bvals.shape[-1] == 4:
             bvecs = bvals[:, 1:]
@@ -227,12 +235,12 @@ def gradient_table(bvals, bvecs=None, big_delta=None, small_delta=None,
             bvals = np.squeeze(bvals[0, :])
         else:
             raise ValueError("input should be bvals and bvecs OR an (N, 4)"
-                             "array containing both bvals and bvecs")
+                             " array containing both bvals and bvecs")
     else:
         bvecs = np.asarray(bvecs)
         if (bvecs.shape[1] > bvecs.shape[0])  and bvecs.shape[0]>1:
             bvecs = bvecs.T
-    return gradient_table_from_bvals_bvecs(bvals, bvecs, big_delta=None,
-                                           small_delta=None,
+    return gradient_table_from_bvals_bvecs(bvals, bvecs, big_delta=big_delta,
+                                           small_delta=small_delta,
                                            b0_threshold=b0_threshold,
-                                           atol=1e-2)
+                                           atol=atol)
