@@ -53,6 +53,7 @@ Hettinger. http://users.rcn.com/python/download/Descriptor.htm
 
 [2] Python data model, http://docs.python.org/reference/datamodel.html
 """
+from __future__ import division, print_function, absolute_import
 
 #-----------------------------------------------------------------------------
 # Classes and Functions
@@ -96,7 +97,7 @@ class ResetMixin(object):
     ...
     ...     @auto_attr
     ...     def y(self):
-    ...         print '*** y computation executed ***'
+    ...         print('*** y computation executed ***')
     ...         return self.x / 2.0
     ...
 
@@ -158,7 +159,7 @@ class OneTimeProperty(object):
           the value of this computation.
         """
         self.getter = func
-        self.name = func.func_name
+        self.name = func.__name__
 
     def __get__(self, obj, type=None):
         """This will be called on attribute access on the class or instance."""
