@@ -7,8 +7,8 @@ docs.  In setup.py in particular, we exec this file, so it cannot import dipy
 # full release.  '.dev' as a _version_extra string means this is a development
 # version
 _version_major = 0
-_version_minor = 9
-_version_micro = 2
+_version_minor = 10
+_version_micro = 1
 #_version_extra = 'dev'
 _version_extra = ''
 
@@ -49,8 +49,7 @@ Current information can always be found from the dipy website - http://dipy.org
 Mailing Lists
 =============
 
-Please see the developer's list at
-http://mail.scipy.org/mailman/listinfo/nipy-devel
+Please see the developer's list at https://mail.python.org/mailman/listinfo/neuroimaging
 
 Code
 ====
@@ -78,7 +77,7 @@ under the GPL license.
 """
 
 # versions for dependencies
-NUMPY_MIN_VERSION='1.6'
+NUMPY_MIN_VERSION='1.7.1'
 SCIPY_MIN_VERSION='0.9'
 CYTHON_MIN_VERSION='0.18'
 NIBABEL_MIN_VERSION='1.2.0'
@@ -86,7 +85,7 @@ NIBABEL_MIN_VERSION='1.2.0'
 # Main setup parameters
 NAME                = 'dipy'
 MAINTAINER          = "Eleftherios Garyfallidis"
-MAINTAINER_EMAIL    = "nipy-devel@neuroimaging.scipy.org"
+MAINTAINER_EMAIL    = "neuroimaging@python.org"
 DESCRIPTION         = description
 LONG_DESCRIPTION    = long_description
 URL                 = "http://dipy.org"
@@ -94,7 +93,7 @@ DOWNLOAD_URL        = "http://github.com/nipy/dipy/archives/master"
 LICENSE             = "BSD license"
 CLASSIFIERS         = CLASSIFIERS
 AUTHOR              = "dipy developers"
-AUTHOR_EMAIL        = "nipy-devel@neuroimaging.scipy.org"
+AUTHOR_EMAIL        = "neuroimaging@python.org"
 PLATFORMS           = "OS Independent"
 MAJOR               = _version_major
 MINOR               = _version_minor
