@@ -15,7 +15,11 @@ class ReconstModel(object):
     """ Abstract class for signal reconstruction models
     """
     def __init__(self, gtab):
-        """
+        """Initialization of the abstract class for signal reconstruction models
+
+        Parameters
+        ----------
+        gtab : GradientTable class instance
 
         """
         self.gtab=gtab
