@@ -37,6 +37,13 @@ class GeneralizedQSamplingModel(OdfModel, Cache):
 
         .. [2] Garyfallidis E, "Towards an accurate brain tractography", PhD
         thesis, University of Cambridge, 2012.
+        
+        Notes
+        -----
+        As of version 0.9, range of the sampling length in GQI2 has changed
+	to match the same scale used in the 'standard' method [1]_. This 
+        means that the value of `sampling_length` should be approximately 
+        1 - 1.3 (see [1]_, pg. 1628). 
 
         Examples
         --------
@@ -49,7 +56,7 @@ class GeneralizedQSamplingModel(OdfModel, Cache):
         >>> from dipy.core.subdivide_octahedron import create_unit_sphere 
         >>> sphere = create_unit_sphere(5)
         >>> from dipy.reconst.gqi import GeneralizedQSamplingModel
-        >>> gq = GeneralizedQSamplingModel(gtab, 'gqi2', 1.4)
+        >>> gq = GeneralizedQSamplingModel(gtab, 'gqi2', 1.1)
         >>> voxel_signal = data[0, 0, 0]
         >>> odf = gq.fit(voxel_signal).odf(sphere)
 
@@ -104,7 +111,7 @@ class GeneralizedQSamplingFit(OdfFit):
             if self.model.method == 'gqi2':
                 H=squared_radial_component
                 #print self.gqi_vector.shape
-                self.gqi_vector = np.real(H(np.dot(self.model.b_vector,                                         sphere.vertices.T) * self.model.Lambda / np.pi))
+                self.gqi_vector = np.real(H(np.dot(self.model.b_vector,                                         sphere.vertices.T) * self.model.Lambda))
             if self.model.method == 'standard':
                 self.gqi_vector = np.real(np.sinc(np.dot(self.model.b_vector,                                   sphere.vertices.T) * self.model.Lambda / np.pi))
             self.model.cache_set('gqi_vector', sphere, self.gqi_vector)
