@@ -4,7 +4,7 @@ from numpy.lib.stride_tricks import as_strided
 
 from ..core.ndindex import ndindex
 from .quick_squash import quick_squash as _squash
-from .base import ReconstModel, ReconstFit
+from .base import ReconstFit
 
 
 def multi_voxel_fit(single_voxel_fit):
@@ -73,6 +73,7 @@ class MultiVoxelFit(ReconstFit):
         S0 = kwargs.get('S0', np.ones(self.fit_array.shape))
         idx = ndindex(self.fit_array.shape)
         ijk = next(idx)
+
         def gimme_S0(S0, ijk):
             if isinstance(S0, np.ndarray):
                 return S0[ijk]
@@ -80,15 +81,24 @@ class MultiVoxelFit(ReconstFit):
                 return S0
 
         kwargs['S0'] = gimme_S0(S0, ijk)
+        # If we have a mask, we might have some Nones up front, skip those:
+        while self.fit_array[ijk] is None:
+            ijk = next(idx)
+
         first_pred = self.fit_array[ijk].predict(*args, **kwargs)
-        result = np.empty(self.fit_array.shape + (first_pred.shape[-1],))
+        result = np.zeros(self.fit_array.shape + (first_pred.shape[-1],))
         result[ijk] = first_pred
         for ijk in idx:
             kwargs['S0'] = gimme_S0(S0, ijk)
-            result[ijk] = self.fit_array[ijk].predict(*args, **kwargs)
+            # If it's masked, we predict a 0:
+            if self.fit_array[ijk] is None:
+                result[ijk] *= 0
+            else:
+                result[ijk] = self.fit_array[ijk].predict(*args, **kwargs)
 
         return result
 
+
 class CallableArray(np.ndarray):
     """An array which can be called like a function"""
     def __call__(self, *args, **kwargs):
