@@ -6,7 +6,9 @@ from __future__ import division, print_function, absolute_import
 import numpy as np
 from nose.tools import (assert_true, assert_equal,
                         assert_almost_equal, assert_raises)
-from numpy.testing import assert_array_equal, assert_array_almost_equal, assert_
+import numpy.testing as npt
+from numpy.testing import (assert_array_equal, assert_array_almost_equal,
+                           assert_)
 import nibabel as nib
 
 import scipy.optimize as opt
@@ -14,21 +16,30 @@ import scipy.optimize as opt
 import dipy.reconst.dti as dti
 from dipy.reconst.dti import (axial_diffusivity, color_fa,
                               fractional_anisotropy, from_lower_triangular,
-                              lower_triangular, mean_diffusivity,
-                              radial_diffusivity, TensorModel, trace,
-                              linearity, planarity, sphericity)
+                              geodesic_anisotropy, lower_triangular,
+                              mean_diffusivity, radial_diffusivity,
+                              TensorModel, trace, linearity, planarity,
+                              sphericity)
 
 from dipy.io.bvectxt import read_bvec_file
 from dipy.data import get_data, dsi_voxels, get_sphere
 
 from dipy.core.subdivide_octahedron import create_unit_sphere
-from dipy.reconst.odf import gfa
 import dipy.core.gradients as grad
 import dipy.core.sphere as dps
 
 from dipy.sims.voxel import single_tensor
 
 
+def test_roll_evals():
+    """
+
+    """
+    # Just making sure this never passes through
+    weird_evals = np.array([1, 0.5])
+    npt.assert_raises(ValueError, dti._roll_evals, weird_evals)
+
+
 def test_tensor_algebra():
     """
     Test that the computation of tensor determinant and norm is correct
@@ -41,20 +52,34 @@ def test_tensor_algebra():
         assert_almost_equal(np.linalg.norm(x), t_norm[i])
 
 
-def test_TensorModel():
-    data, gtab = dsi_voxels()
-    dm = dti.TensorModel(gtab, 'LS')
-    dtifit = dm.fit(data[0, 0, 0])
-    assert_equal(dtifit.fa < 0.5, True)
-    dm = dti.TensorModel(gtab, 'WLS')
-    dtifit = dm.fit(data[0, 0, 0])
-    assert_equal(dtifit.fa < 0.5, True)
-    sphere = create_unit_sphere(4)
-    assert_equal(len(dtifit.odf(sphere)), len(sphere.vertices))
-    assert_almost_equal(dtifit.fa, gfa(dtifit.odf(sphere)), 1)
+def test_tensor_model():
+    fdata, fbval, fbvec = get_data('small_25')
+    data1 = nib.load(fdata).get_data()
+    gtab1 = grad.gradient_table(fbval, fbvec)
+    data2, gtab2 = dsi_voxels()
+    for data, gtab in zip([data1, data2], [gtab1, gtab2]):
+        dm = dti.TensorModel(gtab, 'LS')
+        dtifit = dm.fit(data[0, 0, 0])
+        assert_equal(dtifit.fa < 0.9, True)
+        dm = dti.TensorModel(gtab, 'WLS')
+        dtifit = dm.fit(data[0, 0, 0])
+        assert_equal(dtifit.fa < 0.9, True)
+        assert_equal(dtifit.fa > 0, True)
+        sphere = create_unit_sphere(4)
+        assert_equal(len(dtifit.odf(sphere)), len(sphere.vertices))
+        # Check that the multivoxel case works:
+        dtifit = dm.fit(data)
+
+        # Check that it works on signal that has already been normalized to S0:
+        dm_to_relative = dti.TensorModel(gtab)
+        if np.any(gtab.b0s_mask):
+            relative_data = (data[0, 0, 0]/np.mean(data[0, 0, 0,
+                                                        gtab.b0s_mask]))
+
+            dtifit_to_relative = dm_to_relative.fit(relative_data)
+            npt.assert_almost_equal(dtifit.fa[0, 0, 0], dtifit_to_relative.fa,
+                                    decimal=3)
 
-    # Check that the multivoxel case works:
-    dtifit = dm.fit(data)
     # And smoke-test that all these operations return sensibly-shaped arrays:
     assert_equal(dtifit.fa.shape, data.shape[:3])
     assert_equal(dtifit.ad.shape, data.shape[:3])
@@ -65,24 +90,35 @@ def test_TensorModel():
     assert_equal(dtifit.linearity.shape, data.shape[:3])
     assert_equal(dtifit.planarity.shape, data.shape[:3])
     assert_equal(dtifit.sphericity.shape, data.shape[:3])
-    
+
+    # Test for the shape of the mask
+    assert_raises(ValueError, dm.fit, np.ones((10, 10, 3)), np.ones((3, 3)))
+
     # Make some synthetic data
     b0 = 1000.
     bvecs, bvals = read_bvec_file(get_data('55dir_grad.bvec'))
     gtab = grad.gradient_table_from_bvals_bvecs(bvals, bvecs.T)
     # The first b value is 0., so we take the second one:
     B = bvals[1]
-    #Scale the eigenvalues and tensor by the B value so the units match
+    # Scale the eigenvalues and tensor by the B value so the units match
     D = np.array([1., 1., 1., 0., 0., 1., -np.log(b0) * B]) / B
     evals = np.array([2., 1., 0.]) / B
     md = evals.mean()
     tensor = from_lower_triangular(D)
     A_squiggle = tensor - (1 / 3.0) * np.trace(tensor) * np.eye(3)
-    mode = 3 * np.sqrt(6) * np.linalg.det(A_squiggle / np.linalg.norm(A_squiggle))
-    evecs = np.linalg.eigh(tensor)[1]
-    #Design Matrix
+    mode = (3 * np.sqrt(6) * np.linalg.det(A_squiggle /
+            np.linalg.norm(A_squiggle)))
+    evals_eigh, evecs_eigh = np.linalg.eigh(tensor)
+    # Sort according to eigen-value from large to small:
+    evecs = evecs_eigh[:, np.argsort(evals_eigh)[::-1]]
+    # Check that eigenvalues and eigenvectors are properly sorted through
+    # that previous operation:
+    for i in range(3):
+        assert_array_almost_equal(np.dot(tensor, evecs[:, i]),
+                                  evals[i] * evecs[:, i])
+    # Design Matrix
     X = dti.design_matrix(gtab)
-    #Signals
+    # Signals
     Y = np.exp(np.dot(X, D))
     assert_almost_equal(Y[0], b0)
     Y.shape = (-1,) + Y.shape
@@ -96,10 +132,24 @@ def test_TensorModel():
         assert_true(tensor_fit.model is tensor_model)
         assert_equal(tensor_fit.shape, Y.shape[:-1])
         assert_array_almost_equal(tensor_fit.evals[0], evals)
+        # Test that the eigenvectors are correct, one-by-one:
+        for i in range(3):
+            # Eigenvectors have intrinsic sign ambiguity
+            # (see
+            # http://prod.sandia.gov/techlib/access-control.cgi/2007/076422.pdf)
+            # so we need to allow for sign flips. One of the following should
+            # always be true:
+            assert_(
+                    np.all(np.abs(tensor_fit.evecs[0][:, i] -
+                                  evecs[:, i]) < 10e-6) or
+                    np.all(np.abs(-tensor_fit.evecs[0][:, i] -
+                                  evecs[:, i]) < 10e-6))
+            # We set a fixed tolerance of 10e-6, similar to array_almost_equal
 
+        err_msg = "Calculation of tensor from Y does not compare to "
+        err_msg += "analytical solution"
         assert_array_almost_equal(tensor_fit.quadratic_form[0], tensor,
-                                  err_msg=\
-        "Calculation of tensor from Y does not compare to analytical solution")
+                                  err_msg=err_msg)
 
         assert_almost_equal(tensor_fit.md[0], md)
         assert_array_almost_equal(tensor_fit.mode, mode, decimal=5)
@@ -112,8 +162,36 @@ def test_TensorModel():
                   gtab,
                   fit_method='crazy_method')
 
+    # Test custom fit tensor method
+    try:
+        model = dti.TensorModel(gtab, fit_method=lambda *args, **kwargs: 42)
+        fit = model.fit_method()
+    except Exception as exc:
+        assert False, "TensorModel should accept custom fit methods: %s" % exc
+    assert fit == 42, "Custom fit method for TensorModel returned %s." % fit
 
-def test_indexing_on_TensorFit():
+    # Test multi-voxel data
+    data = np.zeros((3, Y.shape[1]))
+    # Normal voxel
+    data[0] = Y
+    # High diffusion voxel, all diffusing weighted signal equal to zero
+    data[1, gtab.b0s_mask] = b0
+    data[1, ~gtab.b0s_mask] = 0
+    # Masked voxel, all data set to zero
+    data[2] = 0.
+
+    tensor_model = dti.TensorModel(gtab)
+    fit = tensor_model.fit(data)
+    assert_array_almost_equal(fit[0].evals, evals)
+
+    # Evals should be high for high diffusion voxel
+    assert_(all(fit[1].evals > evals[0] * .9))
+
+    # Evals should be zero where data is masked
+    assert_array_almost_equal(fit[2].evals, 0.)
+
+
+def test_indexing_on_tensor_fit():
     params = np.zeros([2, 3, 4, 12])
     fit = dti.TensorFit(None, params)
 
@@ -139,6 +217,12 @@ def test_fa_of_zero():
     assert_array_equal(fa, 0)
 
 
+def test_ga_of_zero():
+    evals = np.zeros((4, 3))
+    ga = geodesic_anisotropy(evals)
+    assert_array_equal(ga, 0)
+
+
 def test_diffusivities():
     psphere = get_sphere('symmetric362')
     bvecs = np.concatenate(([[0, 0, 0]], psphere.vertices))
@@ -146,9 +230,9 @@ def test_diffusivities():
     bvals[0] = 0
     gtab = grad.gradient_table(bvals, bvecs)
     mevals = np.array(([0.0015, 0.0003, 0.0001], [0.0015, 0.0003, 0.0003]))
-    mevecs = [ np.array( [ [1, 0, 0], [0, 1, 0], [0, 0, 1] ] ),
-               np.array( [ [0, 0, 1], [0, 1, 0], [1, 0, 0] ] ) ]
-    S = single_tensor( gtab, 100, mevals[0], mevecs[0], snr=None )
+    mevecs = [np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]]),
+              np.array([[0, 0, 1], [0, 1, 0], [1, 0, 0]])]
+    S = single_tensor(gtab, 100, mevals[0], mevecs[0], snr=None)
 
     dm = dti.TensorModel(gtab, 'LS')
     dmfit = dm.fit(S)
@@ -160,7 +244,7 @@ def test_diffusivities():
     lin = linearity(dmfit.evals)
     plan = planarity(dmfit.evals)
     spher = sphericity(dmfit.evals)
-    
+
     assert_almost_equal(md, (0.0015 + 0.0003 + 0.0001) / 3)
     assert_almost_equal(Trace, (0.0015 + 0.0003 + 0.0001))
     assert_almost_equal(ad, 0.0015)
@@ -177,8 +261,11 @@ def test_color_fa():
     fa = fractional_anisotropy(dmfit.evals)
     cfa = color_fa(fa, dmfit.evecs)
 
-    # evecs should be of shape (fa, 3, 3)
     fa = np.ones((3, 3, 3))
+    # evecs should be of shape (fa, 3, 3)
+    evecs = np.zeros(fa.shape + (3, 2))
+    npt.assert_raises(ValueError, color_fa, fa, evecs)
+
     evecs = np.zeros(fa.shape + (3, 3))
     evecs[..., :, :] = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]])
 
@@ -216,7 +303,7 @@ def test_color_fa():
     assert_array_equal(cfa, true_cfa)
 
 
-def test_WLS_and_LS_fit():
+def test_wls_and_ls_fit():
     """
     Tests the WLS and LS fitting functions to see if they returns the correct
     eigenvalues and eigenvectors.
@@ -226,35 +313,39 @@ def test_WLS_and_LS_fit():
 
     """
 
-    ### Defining Test Voxel (avoid nibabel dependency) ###
+    # Defining Test Voxel (avoid nibabel dependency) ###
 
-    #Recall: D = [Dxx,Dyy,Dzz,Dxy,Dxz,Dyz,log(S_0)] and D ~ 10^-4 mm^2 /s
+    # Recall: D = [Dxx,Dyy,Dzz,Dxy,Dxz,Dyz,log(S_0)] and D ~ 10^-4 mm^2 /s
     b0 = 1000.
     bvec, bval = read_bvec_file(get_data('55dir_grad.bvec'))
     B = bval[1]
-    #Scale the eigenvalues and tensor by the B value so the units match
+    # Scale the eigenvalues and tensor by the B value so the units match
     D = np.array([1., 1., 1., 0., 0., 1., -np.log(b0) * B]) / B
     evals = np.array([2., 1., 0.]) / B
     md = evals.mean()
     tensor = from_lower_triangular(D)
-    #Design Matrix
+    # Design Matrix
     gtab = grad.gradient_table(bval, bvec)
     X = dti.design_matrix(gtab)
-    #Signals
+    # Signals
     Y = np.exp(np.dot(X, D))
     assert_almost_equal(Y[0], b0)
     Y.shape = (-1,) + Y.shape
 
+    # Testing WLS Fit on Single Voxel
+    # If you do something wonky (passing min_signal<0), you should get an
+    # error:
+    npt.assert_raises(ValueError, TensorModel, gtab, fit_method='WLS',
+                      min_signal=-1)
 
-    ### Testing WLS Fit on Single Voxel ###
-    #Estimate tensor from test signals
-    model = TensorModel(gtab, min_signal=1e-8, fit_method='WLS')
+    # Estimate tensor from test signals
+    model = TensorModel(gtab, fit_method='WLS')
     tensor_est = model.fit(Y)
     assert_equal(tensor_est.shape, Y.shape[:-1])
     assert_array_almost_equal(tensor_est.evals[0], evals)
     assert_array_almost_equal(tensor_est.quadratic_form[0], tensor,
                               err_msg="Calculation of tensor from Y does not "
-                                       "compare to analytical solution")
+                                      "compare to analytical solution")
     assert_almost_equal(tensor_est.md[0], md)
 
     # Test that we can fit a single voxel's worth of data (a 1d array)
@@ -267,7 +358,7 @@ def test_WLS_and_LS_fit():
     assert_array_almost_equal(tensor_est.lower_triangular(b0), D)
 
     # Test using fit_method='LS'
-    model = TensorModel(gtab, min_signal=1e-8, fit_method='LS')
+    model = TensorModel(gtab, fit_method='LS')
     tensor_est = model.fit(y)
     assert_equal(tensor_est.shape, tuple())
     assert_array_almost_equal(tensor_est.evals, evals)
@@ -279,8 +370,7 @@ def test_WLS_and_LS_fit():
     assert_array_almost_equal(tensor_est.sphericity, sphericity(evals))
 
 
-
-def test_masked_array_with_Tensor():
+def test_masked_array_with_tensor():
     data = np.ones((2, 4, 56))
     mask = np.array([[True, False, False, True],
                      [True, False, True, False]])
@@ -288,7 +378,7 @@ def test_masked_array_with_Tensor():
     bvec, bval = read_bvec_file(get_data('55dir_grad.bvec'))
     gtab = grad.gradient_table_from_bvals_bvecs(bval, bvec.T)
 
-    tensor_model = TensorModel(gtab, min_signal=1e-9)
+    tensor_model = TensorModel(gtab)
     tensor = tensor_model.fit(data, mask=mask)
     assert_equal(tensor.shape, (2, 4))
     assert_equal(tensor.fa.shape, (2, 4))
@@ -313,8 +403,8 @@ def test_fit_method_error():
     bvec, bval = read_bvec_file(get_data('55dir_grad.bvec'))
     gtab = grad.gradient_table_from_bvals_bvecs(bval, bvec.T)
 
-    # This should work
-    tensor_model = TensorModel(gtab, fit_method='WLS')
+    # This should work (smoke-testing!):
+    TensorModel(gtab, fit_method='WLS')
 
     # This should raise an error because there is no such fit_method
     assert_raises(ValueError, TensorModel, gtab, min_signal=1e-9,
@@ -355,16 +445,23 @@ def test_from_lower_triangular():
 
 
 def test_all_constant():
-    """
-
-    """
     bvecs, bvals = read_bvec_file(get_data('55dir_grad.bvec'))
     gtab = grad.gradient_table_from_bvals_bvecs(bvals, bvecs.T)
-    fit_methods = ['LS', 'OLS', 'NNLS']
+    fit_methods = ['LS', 'OLS', 'NNLS', 'RESTORE']
     for fit_method in fit_methods:
         dm = dti.TensorModel(gtab)
-        assert_almost_equal(dm.fit(np.zeros(bvals.shape[0])).fa, 0)
         assert_almost_equal(dm.fit(100 * np.ones(bvals.shape[0])).fa, 0)
+        # Doesn't matter if the signal is smaller than 1:
+        assert_almost_equal(dm.fit(0.4 * np.ones(bvals.shape[0])).fa, 0)
+
+
+def test_all_zeros():
+    bvecs, bvals = read_bvec_file(get_data('55dir_grad.bvec'))
+    gtab = grad.gradient_table_from_bvals_bvecs(bvals, bvecs.T)
+    fit_methods = ['LS', 'OLS', 'NNLS', 'RESTORE']
+    for fit_method in fit_methods:
+        dm = dti.TensorModel(gtab)
+        assert_array_almost_equal(dm.fit(np.zeros(bvals.shape[0])).evals, 0)
 
 
 def test_mask():
@@ -383,21 +480,21 @@ def test_mask():
     # Except for the one voxel that was selected by the mask:
     assert_almost_equal(dtifit_w_mask.fa[0, 0, 0], dtifit.fa[0, 0, 0])
 
+
 def test_nnls_jacobian_fucn():
     b0 = 1000.
     bvecs, bval = read_bvec_file(get_data('55dir_grad.bvec'))
     gtab = grad.gradient_table(bval, bvecs)
     B = bval[1]
 
-    #Scale the eigenvalues and tensor by the B value so the units match
+    # Scale the eigenvalues and tensor by the B value so the units match
     D = np.array([1., 1., 1., 0., 0., 1., -np.log(b0) * B]) / B
-    evals = np.array([2., 1., 0.]) / B
 
-    #Design Matrix
+    # Design Matrix
     X = dti.design_matrix(gtab)
 
-    #Signals
-    Y = np.exp(np.dot(X,D))
+    # Signals
+    Y = np.exp(np.dot(X, D))
 
     # Test Jacobian at D
     args = [X, Y]
@@ -416,114 +513,136 @@ def test_nnls_jacobian_fucn():
         approx = opt.approx_fprime(D, dti._nlls_err_func, 1e-8, *args)
         assert_true(np.allclose(approx, analytical[i]))
 
+
 def test_nlls_fit_tensor():
-     """
-     Test the implementation of NLLS and RESTORE
-     """
+    """
+    Test the implementation of NLLS and RESTORE
+    """
 
-     b0 = 1000.
-     bvecs, bval = read_bvec_file(get_data('55dir_grad.bvec'))
-     gtab = grad.gradient_table(bval, bvecs)
-     B = bval[1]
+    b0 = 1000.
+    bvecs, bval = read_bvec_file(get_data('55dir_grad.bvec'))
+    gtab = grad.gradient_table(bval, bvecs)
+    B = bval[1]
 
-     #Scale the eigenvalues and tensor by the B value so the units match
-     D = np.array([1., 1., 1., 0., 0., 1., -np.log(b0) * B]) / B
-     evals = np.array([2., 1., 0.]) / B
-     md = evals.mean()
-     tensor = from_lower_triangular(D)
+    # Scale the eigenvalues and tensor by the B value so the units match
+    D = np.array([1., 1., 1., 0., 0., 1., -np.log(b0) * B]) / B
+    evals = np.array([2., 1., 0.]) / B
+    md = evals.mean()
+    tensor = from_lower_triangular(D)
 
-     #Design Matrix
-     X = dti.design_matrix(gtab)
+    # Design Matrix
+    X = dti.design_matrix(gtab)
 
-     #Signals
-     Y = np.exp(np.dot(X,D))
-     Y.shape = (-1,) + Y.shape
+    # Signals
+    Y = np.exp(np.dot(X, D))
+    Y.shape = (-1,) + Y.shape
 
-     #Estimate tensor from test signals and compare against expected result
-     #using non-linear least squares:
-     tensor_model = dti.TensorModel(gtab, fit_method='NLLS')
-     tensor_est = tensor_model.fit(Y)
-     assert_equal(tensor_est.shape, Y.shape[:-1])
-     assert_array_almost_equal(tensor_est.evals[0], evals)
-     assert_array_almost_equal(tensor_est.quadratic_form[0], tensor)
-     assert_almost_equal(tensor_est.md[0], md)
+    # Estimate tensor from test signals and compare against expected result
+    # using non-linear least squares:
+    tensor_model = dti.TensorModel(gtab, fit_method='NLLS')
+    tensor_est = tensor_model.fit(Y)
+    assert_equal(tensor_est.shape, Y.shape[:-1])
+    assert_array_almost_equal(tensor_est.evals[0], evals)
+    assert_array_almost_equal(tensor_est.quadratic_form[0], tensor)
+    assert_almost_equal(tensor_est.md[0], md)
 
-     # Using the gmm weighting scheme:
-     tensor_model = dti.TensorModel(gtab, fit_method='NLLS', weighting='gmm')
-     assert_equal(tensor_est.shape, Y.shape[:-1])
-     assert_array_almost_equal(tensor_est.evals[0], evals)
-     assert_array_almost_equal(tensor_est.quadratic_form[0], tensor)
-     assert_almost_equal(tensor_est.md[0], md)
+    # You can also do this without the Jacobian (though it's slower):
+    tensor_model = dti.TensorModel(gtab, fit_method='NLLS', jac=False)
+    tensor_est = tensor_model.fit(Y)
+    assert_equal(tensor_est.shape, Y.shape[:-1])
+    assert_array_almost_equal(tensor_est.evals[0], evals)
+    assert_array_almost_equal(tensor_est.quadratic_form[0], tensor)
+    assert_almost_equal(tensor_est.md[0], md)
 
-     # Use NLLS with some actual 4D data:
-     data, bvals, bvecs = get_data('small_25')
-     gtab = grad.gradient_table(bvals, bvecs)
-     tm1 = dti.TensorModel(gtab, fit_method='NLLS')
-     dd = nib.load(data).get_data()
-     tf1 = tm1.fit(dd)
-     tm2 = dti.TensorModel(gtab)
-     tf2 = tm2.fit(dd)
+    # Using the gmm weighting scheme:
+    tensor_model = dti.TensorModel(gtab, fit_method='NLLS', weighting='gmm')
+    tensor_est = tensor_model.fit(Y)
+    assert_equal(tensor_est.shape, Y.shape[:-1])
+    assert_array_almost_equal(tensor_est.evals[0], evals)
+    assert_array_almost_equal(tensor_est.quadratic_form[0], tensor)
+    assert_almost_equal(tensor_est.md[0], md)
+
+    # If you use sigma weighting, you'd better provide a sigma:
+    tensor_model = dti.TensorModel(gtab, fit_method='NLLS', weighting='sigma')
+    npt.assert_raises(ValueError, tensor_model.fit, Y)
+
+    # Use NLLS with some actual 4D data:
+    data, bvals, bvecs = get_data('small_25')
+    gtab = grad.gradient_table(bvals, bvecs)
+    tm1 = dti.TensorModel(gtab, fit_method='NLLS')
+    dd = nib.load(data).get_data()
+    tf1 = tm1.fit(dd)
+    tm2 = dti.TensorModel(gtab)
+    tf2 = tm2.fit(dd)
+
+    assert_array_almost_equal(tf1.fa, tf2.fa, decimal=1)
 
-     assert_array_almost_equal(tf1.fa, tf2.fa, decimal=1)
 
 def test_restore():
-     """
-     Test the implementation of the RESTORE algorithm
-     """
-     b0 = 1000.
-     bvecs, bval = read_bvec_file(get_data('55dir_grad.bvec'))
-     gtab = grad.gradient_table(bval, bvecs)
-     B = bval[1]
+    """
+    Test the implementation of the RESTORE algorithm
+    """
+    b0 = 1000.
+    bvecs, bval = read_bvec_file(get_data('55dir_grad.bvec'))
+    gtab = grad.gradient_table(bval, bvecs)
+    B = bval[1]
 
-     #Scale the eigenvalues and tensor by the B value so the units match
-     D = np.array([1., 1., 1., 0., 0., 1., -np.log(b0) * B]) / B
-     evals = np.array([2., 1., 0.]) / B
-     md = evals.mean()
-     tensor = from_lower_triangular(D)
+    # Scale the eigenvalues and tensor by the B value so the units match
+    D = np.array([1., 1., 1., 0., 0., 1., -np.log(b0) * B]) / B
+    evals = np.array([2., 1., 0.]) / B
+    tensor = from_lower_triangular(D)
 
-     #Design Matrix
-     X = dti.design_matrix(gtab)
+    # Design Matrix
+    X = dti.design_matrix(gtab)
 
-     #Signals
-     Y = np.exp(np.dot(X,D))
-     Y.shape = (-1,) + Y.shape
-     for drop_this in range(1, Y.shape[-1]):
-         # RESTORE estimates should be robust to dropping
-         this_y = Y.copy()
-         this_y[:, drop_this] = 1.0
-         tensor_model = dti.TensorModel(gtab, fit_method='restore',
-                                        sigma=67.0)
+    # Signals
+    Y = np.exp(np.dot(X, D))
+    Y.shape = (-1,) + Y.shape
+    for drop_this in range(1, Y.shape[-1]):
+        for jac in [True, False]:
+            # RESTORE estimates should be robust to dropping
+            this_y = Y.copy()
+            this_y[:, drop_this] = 1.0
+            for sigma in [67.0, np.ones(this_y.shape[-1]) * 67.0]:
+                tensor_model = dti.TensorModel(gtab, fit_method='restore',
+                                               jac=jac,
+                                               sigma=67.0)
+
+                tensor_est = tensor_model.fit(this_y)
+                assert_array_almost_equal(tensor_est.evals[0], evals,
+                                          decimal=3)
+                assert_array_almost_equal(tensor_est.quadratic_form[0], tensor,
+                                          decimal=3)
+
+    # If sigma is very small, it still needs to work:
+    tensor_model = dti.TensorModel(gtab, fit_method='restore', sigma=0.0001)
+    tensor_model.fit(Y.copy())
 
-         tensor_est = tensor_model.fit(this_y)
-         assert_array_almost_equal(tensor_est.evals[0], evals, decimal=3)
-         assert_array_almost_equal(tensor_est.quadratic_form[0], tensor,
-                                   decimal=3)
 
 def test_adc():
     """
-    Test the implementation of the calculation of apparent diffusion coefficient
+    Test the implementation of the calculation of apparent diffusion
+    coefficient
     """
     data, gtab = dsi_voxels()
     dm = dti.TensorModel(gtab, 'LS')
     mask = np.zeros(data.shape[:-1], dtype=bool)
     mask[0, 0, 0] = True
     dtifit = dm.fit(data)
-    sphere = create_unit_sphere(4)
-    
     # The ADC in the principal diffusion direction should be equal to the AD in
     # each voxel:
 
-    pdd0 = dtifit.evecs[0,0,0,0]
+    pdd0 = dtifit.evecs[0, 0, 0, 0]
     sphere_pdd0 = dps.Sphere(x=pdd0[0], y=pdd0[1], z=pdd0[2])
-    assert_array_almost_equal(dtifit.adc(sphere_pdd0)[0,0,0],
-                            dtifit.ad[0,0,0], decimal=5)
-        
-    
+    assert_array_almost_equal(dtifit.adc(sphere_pdd0)[0, 0, 0],
+                              dtifit.ad[0, 0, 0], decimal=5)
+
     # Test that it works for cases in which the data is 1D
-    dtifit = dm.fit(data[0,0,0])
+    dtifit = dm.fit(data[0, 0, 0])
     sphere_pdd0 = dps.Sphere(x=pdd0[0], y=pdd0[1], z=pdd0[2])
     assert_array_almost_equal(dtifit.adc(sphere_pdd0),
-                        dtifit.ad, decimal=5)
+                              dtifit.ad, decimal=5)
+
 
 def test_predict():
     """
@@ -535,18 +654,41 @@ def test_predict():
     bvals[0] = 0
     gtab = grad.gradient_table(bvals, bvecs)
     mevals = np.array(([0.0015, 0.0003, 0.0001], [0.0015, 0.0003, 0.0003]))
-    mevecs = [ np.array( [ [1, 0, 0], [0, 1, 0], [0, 0, 1] ] ),
-               np.array( [ [0, 0, 1], [0, 1, 0], [1, 0, 0] ] ) ]
-    S = single_tensor( gtab, 100, mevals[0], mevecs[0], snr=None )
+    mevecs = [np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]]),
+              np.array([[0, 0, 1], [0, 1, 0], [1, 0, 0]])]
+    S = single_tensor(gtab, 100, mevals[0], mevecs[0], snr=None)
 
     dm = dti.TensorModel(gtab, 'LS')
     dmfit = dm.fit(S)
-    
     assert_array_almost_equal(dmfit.predict(gtab, S0=100), S)
+    assert_array_almost_equal(dm.predict(dmfit.model_params, S0=100), S)
 
-    data, gtab = dsi_voxels()
+    fdata, fbvals, fbvecs = get_data()
+    data = nib.load(fdata).get_data()
+    # Make the data cube a bit larger:
+    data = np.tile(data.T, 2).T
+    gtab = grad.gradient_table(fbvals, fbvecs)
     dtim = dti.TensorModel(gtab)
     dtif = dtim.fit(data)
-    S0 = np.mean(data[...,gtab.b0s_mask], -1)
+    S0 = np.mean(data[..., gtab.b0s_mask], -1)
     p = dtif.predict(gtab, S0)
+    assert_equal(p.shape, data.shape)
 
+
+def test_eig_from_lo_tri():
+    psphere = get_sphere('symmetric362')
+    bvecs = np.concatenate(([[0, 0, 0]], psphere.vertices))
+    bvals = np.zeros(len(bvecs)) + 1000
+    bvals[0] = 0
+    gtab = grad.gradient_table(bvals, bvecs)
+    mevals = np.array(([0.0015, 0.0003, 0.0001], [0.0015, 0.0003, 0.0003]))
+    mevecs = [np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]]),
+              np.array([[0, 0, 1], [0, 1, 0], [1, 0, 0]])]
+    S = np.array([[single_tensor(gtab, 100, mevals[0], mevecs[0], snr=None),
+                   single_tensor(gtab, 100, mevals[0], mevecs[0], snr=None)]])
+
+    dm = dti.TensorModel(gtab, 'LS')
+    dmfit = dm.fit(S)
+
+    lo_tri = lower_triangular(dmfit.quadratic_form)
+    assert_array_almost_equal(dti.eig_from_lo_tri(lo_tri), dmfit.model_params)
