@@ -117,8 +117,8 @@ def draw_lattice_2d(nrows, ncols, delta):
 
 
 def plot_2d_diffeomorphic_map(mapping, delta=10, fname=None,
-                              direct_grid_shape=None, direct_grid_affine=-1,
-                              inverse_grid_shape=None, inverse_grid_affine=-1,
+                              direct_grid_shape=None, direct_grid2world=-1,
+                              inverse_grid_shape=None, inverse_grid2world=-1,
                               show_figure=True):
     r"""Draw the effect of warping a regular lattice by a diffeomorphic map.
 
@@ -145,12 +145,12 @@ def plot_2d_diffeomorphic_map(mapping, delta=10, fname=None,
         equal to the shape of the fixed image. In other words, the resulting
         deformed grid (deformed by the direct transformation) will normally
         have the same shape as the fixed image.
-    direct_grid_affine : array, shape (3, 3), optional
+    direct_grid2world : array, shape (3, 3), optional
         the affine transformation mapping the direct grid's coordinates to
         physical space. By default, this transformation will correspond to
         the image-to-world transformation corresponding to the default
         direct_grid_shape (in general, if users specify a direct_grid_shape,
-        they should also specify direct_grid_affine).
+        they should also specify direct_grid2world).
     inverse_grid_shape : tuple, shape (2,), optional
         the shape of the grid image after being deformed by the inverse
         transformation. By default, the shape of the deformed grid under the
@@ -158,12 +158,12 @@ def plot_2d_diffeomorphic_map(mapping, delta=10, fname=None,
         the diffeomorphic map was generated by a registration algorithm
         (so it corresponds to the effect of warping the static image towards
         the moving).
-    inverse_grid_affine : array, shape (3, 3), optional
+    inverse_grid2world : array, shape (3, 3), optional
         the affine transformation mapping inverse grid's coordinates to
         physical space. By default, this transformation will correspond to
         the image-to-world transformation corresponding to the default
         inverse_grid_shape (in general, if users specify an inverse_grid_shape,
-        they should also specify inverse_grid_affine).
+        they should also specify inverse_grid2world).
     show_figure : bool, optional
         if True (default), the deformed grids will be ploted using matplotlib,
         else the grids are just returned
@@ -191,32 +191,32 @@ def plot_2d_diffeomorphic_map(mapping, delta=10, fname=None,
         # By default, direct_grid_shape is the codomain grid
         if direct_grid_shape is None:
             direct_grid_shape = mapping.codomain_shape
-        if direct_grid_affine is -1:
-            direct_grid_affine = mapping.codomain_affine
+        if direct_grid2world is -1:
+            direct_grid2world = mapping.codomain_grid2world
 
         # By default, the inverse grid is the domain grid
         if inverse_grid_shape is None:
             inverse_grid_shape = mapping.domain_shape
-        if inverse_grid_affine is -1:
-            inverse_grid_affine = mapping.domain_affine
+        if inverse_grid2world is -1:
+            inverse_grid2world = mapping.domain_grid2world
     else:
         # Now by default, direct_grid_shape is the mapping's input grid
         if direct_grid_shape is None:
             direct_grid_shape = mapping.domain_shape
-        if direct_grid_affine is -1:
-            direct_grid_affine = mapping.domain_affine
+        if direct_grid2world is -1:
+            direct_grid2world = mapping.domain_grid2world
 
         # By default, the output grid is the mapping's domain grid
         if inverse_grid_shape is None:
             inverse_grid_shape = mapping.codomain_shape
-        if inverse_grid_affine is -1:
-            inverse_grid_affine = mapping.codomain_affine
+        if inverse_grid2world is -1:
+            inverse_grid2world = mapping.codomain_grid2world
 
     # The world-to-image (image = drawn lattice on the output grid)
     # transformation is the inverse of the output affine
     world_to_image = None
-    if inverse_grid_affine is not None:
-        world_to_image = np.linalg.inv(inverse_grid_affine)
+    if inverse_grid2world is not None:
+        world_to_image = np.linalg.inv(inverse_grid2world)
 
     # Draw the squares on the output grid
     lattice_out = draw_lattice_2d(
@@ -227,13 +227,13 @@ def plot_2d_diffeomorphic_map(mapping, delta=10, fname=None,
 
     # Warp in the forward direction (sampling it on the input grid)
     warped_forward = mapping.transform(lattice_out, 'linear', world_to_image,
-                                       direct_grid_shape, direct_grid_affine)
+                                       direct_grid_shape, direct_grid2world)
 
     # Now, the world-to-image (image = drawn lattice on the input grid)
     # transformation is the inverse of the input affine
     world_to_image = None
-    if direct_grid_affine is not None:
-        world_to_image = np.linalg.inv(direct_grid_affine)
+    if direct_grid2world is not None:
+        world_to_image = np.linalg.inv(direct_grid2world)
 
     # Draw the squares on the input grid
     lattice_in = draw_lattice_2d((direct_grid_shape[0] + delta) // (delta + 1),
@@ -244,7 +244,7 @@ def plot_2d_diffeomorphic_map(mapping, delta=10, fname=None,
     # Warp in the backward direction (sampling it on the output grid)
     warped_backward = mapping.transform_inverse(
         lattice_in, 'linear', world_to_image, inverse_grid_shape,
-        inverse_grid_affine)
+        inverse_grid2world)
 
     # Now plot the grids
     if show_figure:
@@ -367,7 +367,6 @@ def overlay_slices(L, R, slice_index=None, slice_type=1, ltitle='Left',
     elif slice_type is 2:
         if slice_index is None:
             slice_index = sh[2]//2
-        slice_index = sh[2]//2
         colorImage = np.zeros(shape=(sh[1], sh[0], 3), dtype=np.uint8)
         ll = np.asarray(L[:, :, slice_index]).astype(np.uint8).T
         rr = np.asarray(R[:, :, slice_index]).astype(np.uint8).T
